#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : int   = open file descriptor
//   in_argv[1] : int   = length of data to read in
//   out_argv[0]: int = error code, 0 on success, nonzero on error
//   out_argv[1]: struct = file trace header
//   out_argv[2]: float vector = file trace data
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int fd; int err = 0; short len = 0; int i;
  segy_trace_header header;
  float *data = NULL;
  mwSize size[2] = {0, 0};
  mxArray *LineSeqNumber, *ReelSeqNumber, *FieldRecordNumber;
  mxArray *TraceNumber, *EnergySourceNumber, *CDPEnsembleNumber, *CDPTraceNumber;
  mxArray *TraceCode, *VertSumTraces, *HorizSumTraces, *DataUse;
  mxArray *SourceGroupDist, *GroupElevation, *SourceElevation, *SourceDepth;
  mxArray *GroupDatumElevation, *SourceDatumElevation, *SourceWaterDepth, *GroupWaterDepth;
  mxArray *ElevationScalar, *CoordinateScalar, *SourceX, *SourceY;
  mxArray *GroupX, *GroupY, *CoordinateUnit, *WeatheringVelocity;
  mxArray *SubweatheringVelocity, *SourceTime, *GroupTime, *SourceCorrection;
  mxArray *GroupCorrection, *TotalCorrection, *LagTimeA, *LagTimeB;
  mxArray *DelayAccordingTime, *BruteTime, *MuteTime, *SamplesPerTrace;
  mxArray *SampleInterval, *GainType, *GainConstant, *InitialGain;
  mxArray *Correlated, *StartSweepFreq, *EndSweepFreq, *SweepLength;
  mxArray *SweepType, *StartSweepTaperLength, *EndSweepTaperLength, *TaperType;
  mxArray *AliasFilterFreq, *AliasFilterSlope, *NotchFilterFreq, *NotchFilterSlope;
  mxArray *LowCutFreq, *HightCutFreq, *LowCutSlope, *HighCutSlope;
  mxArray *Year, *DayOfYear, *Hour, *Minute;
  mxArray *Second, *TimeCode, *TraceWeightFactor, *GroupRollNumber;
  mxArray *GroupTraceStart, *GroupTraceEnd, *GapSize, *Overtravel;
  mxArray *CDPEnsembleX, *CDPEnsembleY, *InLine, *CrossLine;
  mxArray *Shotpoint, *ShotpointScalar, *MeasurementUnit, *TransductionConstant;
  mxArray *TransductionUnit, *DeviceID, *TimeScalar, *SourceType;
  mxArray *SourceDirection, *SourceMeasurement, *SourceMeasurementUnit, *unassigned;

  // check input arguments
  if (in_argc < 2 || ! mxIsInt32(in_argv[0]) || mxIsEmpty(in_argv[0]) ||
      ! mxIsNumeric(in_argv[1]) || mxIsEmpty(in_argv[1])) {
    mexPrintf("segy_read_trace: input is int32 file descriptor, length\n");
    return;
  }
  fd = ((int*)mxGetPr(in_argv[0]))[0];
  len = (int)mxGetScalar(in_argv[1]);
  data = (float*)malloc(sizeof(float) * len);

  // check output arguments
  if (out_argc < 3) {
    mexPrintf("segy_read_trace: must have three output arguments\n");
    return;
  }
  
  // call function, if error return immeadiately
  err = segy_read_trace(fd, 1, &header, data, len);

  // store error return
  out_argv[0] = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (out_argv[0]) ((int*)mxGetPr(out_argv[0]))[0] = err;
  else {
    mexPrintf("segy_read_file_header: out of memory\n");
    return;
  }

  if (err) {
    // store empty struct array
    size[0] = 1; size[1] = 1;
    out_argv[1] = mxCreateStructArray(2, size, 0, NULL);
    if (! out_argv[1]) {
      mexPrintf("segy_read_trace: out of memory\n");
      return;
    }

    // store zeroed data array
    out_argv[2] = mxCreateNumericMatrix(len,1,mxSINGLE_CLASS,mxREAL);
    if (! out_argv[2]) {
      mexPrintf("segy_read_trace: out of memory\n");
      return;
    }

    free(data);
    return;
  }

  // create output MATLAB data array
  out_argv[2] = mxCreateNumericMatrix(len,1,mxSINGLE_CLASS,mxREAL);
  if (! out_argv[2]) {
    mexPrintf("segy_read_trace: out of memory\n");
    return;
  }

  // store data in data array
  for (i=0; i<len; i++) ((float*)mxGetPr(out_argv[2]))[i] = data[i];
  free(data);

  // create output MATLAB struct
  size[0] = 1; size[1] = 1;
  out_argv[1] = mxCreateStructArray(2, size, 87, segy_trace_header_names);
  if (! out_argv[1]) {
    mexPrintf("segy_read_trace: out of memory\n");
    return;
  }

  // copy C struct field values to MATLAB struct fields

  LineSeqNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(LineSeqNumber)) ((int*)mxGetData(LineSeqNumber))[0] = header.LineSeqNumber;
  mxSetField(out_argv[1], 0, "LineSeqNumber", LineSeqNumber);

  ReelSeqNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(ReelSeqNumber)) ((int*)mxGetData(ReelSeqNumber))[0] = header.ReelSeqNumber;
  mxSetField(out_argv[1], 0, "ReelSeqNumber", ReelSeqNumber);

  FieldRecordNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(FieldRecordNumber)) ((int*)mxGetData(FieldRecordNumber))[0] = header.FieldRecordNumber;
  mxSetField(out_argv[1], 0, "FieldRecordNumber", FieldRecordNumber);

  TraceNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(TraceNumber)) ((int*)mxGetData(TraceNumber))[0] = header.TraceNumber;
  mxSetField(out_argv[1], 0, "TraceNumber", TraceNumber);

  EnergySourceNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(EnergySourceNumber)) ((int*)mxGetData(EnergySourceNumber))[0] = header.EnergySourceNumber;
  mxSetField(out_argv[1], 0, "EnergySourceNumber", EnergySourceNumber);

  CDPEnsembleNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(CDPEnsembleNumber)) ((int*)mxGetData(CDPEnsembleNumber))[0] = header.CDPEnsembleNumber;
  mxSetField(out_argv[1], 0, "CDPEnsembleNumber", CDPEnsembleNumber);

  CDPTraceNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(CDPTraceNumber)) ((int*)mxGetData(CDPTraceNumber))[0] = header.CDPTraceNumber;
  mxSetField(out_argv[1], 0, "CDPTraceNumber", CDPTraceNumber);

  TraceCode = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TraceCode)) ((short*)mxGetData(TraceCode))[0] = header.TraceCode;
  mxSetField(out_argv[1], 0, "TraceCode", TraceCode);

  VertSumTraces = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(VertSumTraces)) ((short*)mxGetData(VertSumTraces))[0] = header.VertSumTraces;
  mxSetField(out_argv[1], 0, "VertSumTraces", VertSumTraces);

  HorizSumTraces = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(HorizSumTraces)) ((short*)mxGetData(HorizSumTraces))[0] = header.HorizSumTraces;
  mxSetField(out_argv[1], 0, "HorizSumTraces", HorizSumTraces);

  DataUse = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(DataUse)) ((short*)mxGetData(DataUse))[0] = header.DataUse;
  mxSetField(out_argv[1], 0, "DataUse", DataUse);

  SourceGroupDist = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(SourceGroupDist)) ((int*)mxGetData(SourceGroupDist))[0] = header.SourceGroupDist;
  mxSetField(out_argv[1], 0, "SourceGroupDist", SourceGroupDist);

  GroupElevation = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(GroupElevation)) ((int*)mxGetData(GroupElevation))[0] = header.GroupElevation;
  mxSetField(out_argv[1], 0, "GroupElevation", GroupElevation);

  SourceElevation = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(SourceElevation)) ((int*)mxGetData(SourceElevation))[0] = header.SourceElevation;
  mxSetField(out_argv[1], 0, "SourceElevation", SourceElevation);

  SourceDepth = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(SourceDepth)) ((int*)mxGetData(SourceDepth))[0] = header.SourceDepth;
  mxSetField(out_argv[1], 0, "SourceDepth", SourceDepth);

  GroupDatumElevation = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(GroupDatumElevation)) ((int*)mxGetData(GroupDatumElevation))[0] = header.GroupDatumElevation;
  mxSetField(out_argv[1], 0, "GroupDatumElevation", GroupDatumElevation);

  SourceDatumElevation = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(SourceDatumElevation)) ((int*)mxGetData(SourceDatumElevation))[0] = header.SourceDatumElevation;
  mxSetField(out_argv[1], 0, "SourceDatumElevation", SourceDatumElevation);

  SourceWaterDepth = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(SourceWaterDepth)) ((int*)mxGetData(SourceWaterDepth))[0] = header.SourceWaterDepth;
  mxSetField(out_argv[1], 0, "SourceWaterDepth", SourceWaterDepth);

  GroupWaterDepth = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(GroupWaterDepth)) ((int*)mxGetData(GroupWaterDepth))[0] = header.GroupWaterDepth;
  mxSetField(out_argv[1], 0, "GroupWaterDepth", GroupWaterDepth);

  ElevationScalar = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(ElevationScalar)) ((short*)mxGetData(ElevationScalar))[0] = header.ElevationScalar;
  mxSetField(out_argv[1], 0, "ElevationScalar", ElevationScalar);

  CoordinateScalar = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(CoordinateScalar)) ((short*)mxGetData(CoordinateScalar))[0] = header.CoordinateScalar;
  mxSetField(out_argv[1], 0, "CoordinateScalar", CoordinateScalar);

  SourceX = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(SourceX)) ((int*)mxGetData(SourceX))[0] = header.SourceX;
  mxSetField(out_argv[1], 0, "SourceX", SourceX);

  SourceY = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(SourceY)) ((int*)mxGetData(SourceY))[0] = header.SourceY;
  mxSetField(out_argv[1], 0, "SourceY", SourceY);

  GroupX = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(GroupX)) ((int*)mxGetData(GroupX))[0] = header.GroupX;
  mxSetField(out_argv[1], 0, "GroupX", GroupX);

  GroupY = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(GroupY)) ((int*)mxGetData(GroupY))[0] = header.GroupY;
  mxSetField(out_argv[1], 0, "GroupY", GroupY);

  CoordinateUnit = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(CoordinateUnit)) ((short*)mxGetData(CoordinateUnit))[0] = header.CoordinateUnit;
  mxSetField(out_argv[1], 0, "CoordinateUnit", CoordinateUnit);

  WeatheringVelocity = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(WeatheringVelocity)) ((short*)mxGetData(WeatheringVelocity))[0] = header.WeatheringVelocity;
  mxSetField(out_argv[1], 0, "WeatheringVelocity", WeatheringVelocity);

  SubweatheringVelocity = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SubweatheringVelocity)) ((short*)mxGetData(SubweatheringVelocity))[0] = header.SubweatheringVelocity;
  mxSetField(out_argv[1], 0, "SubweatheringVelocity", SubweatheringVelocity);

  SourceTime = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SourceTime)) ((short*)mxGetData(SourceTime))[0] = header.SourceTime;
  mxSetField(out_argv[1], 0, "SourceTime", SourceTime);

  GroupTime = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GroupTime)) ((short*)mxGetData(GroupTime))[0] = header.GroupTime;
  mxSetField(out_argv[1], 0, "GroupTime", GroupTime);

  SourceCorrection = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SourceCorrection)) ((short*)mxGetData(SourceCorrection))[0] = header.SourceCorrection;
  mxSetField(out_argv[1], 0, "SourceCorrection", SourceCorrection);

  GroupCorrection = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GroupCorrection)) ((short*)mxGetData(GroupCorrection))[0] = header.GroupCorrection;
  mxSetField(out_argv[1], 0, "GroupCorrection", GroupCorrection);

  TotalCorrection = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TotalCorrection)) ((short*)mxGetData(TotalCorrection))[0] = header.TotalCorrection;
  mxSetField(out_argv[1], 0, "TotalCorrection", TotalCorrection);

  LagTimeA = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(LagTimeA)) ((short*)mxGetData(LagTimeA))[0] = header.LagTimeA;
  mxSetField(out_argv[1], 0, "LagTimeA", LagTimeA);

  LagTimeB = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(LagTimeB)) ((short*)mxGetData(LagTimeB))[0] = header.LagTimeB;
  mxSetField(out_argv[1], 0, "LagTimeB", LagTimeB);

  DelayAccordingTime = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(DelayAccordingTime)) ((short*)mxGetData(DelayAccordingTime))[0] = header.DelayAccordingTime;
  mxSetField(out_argv[1], 0, "DelayAccordingTime", DelayAccordingTime);

  BruteTime = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(BruteTime)) ((short*)mxGetData(BruteTime))[0] = header.BruteTime;
  mxSetField(out_argv[1], 0, "BruteTime", BruteTime);

  MuteTime = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(MuteTime)) ((short*)mxGetData(MuteTime))[0] = header.MuteTime;
  mxSetField(out_argv[1], 0, "MuteTime", MuteTime);

  SamplesPerTrace = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SamplesPerTrace)) ((short*)mxGetData(SamplesPerTrace))[0] = header.SamplesPerTrace;
  mxSetField(out_argv[1], 0, "SamplesPerTrace", SamplesPerTrace);

  SampleInterval = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SampleInterval)) ((short*)mxGetData(SampleInterval))[0] = header.SampleInterval;
  mxSetField(out_argv[1], 0, "SampleInterval", SampleInterval);

  GainType = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GainType)) ((short*)mxGetData(GainType))[0] = header.GainType;
  mxSetField(out_argv[1], 0, "GainType", GainType);

  GainConstant = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GainConstant)) ((short*)mxGetData(GainConstant))[0] = header.GainConstant;
  mxSetField(out_argv[1], 0, "GainConstant", GainConstant);

  InitialGain = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(InitialGain)) ((short*)mxGetData(InitialGain))[0] = header.InitialGain;
  mxSetField(out_argv[1], 0, "InitialGain", InitialGain);

  Correlated = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(Correlated)) ((short*)mxGetData(Correlated))[0] = header.Correlated;
  mxSetField(out_argv[1], 0, "Correlated", Correlated);

  StartSweepFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(StartSweepFreq)) ((short*)mxGetData(StartSweepFreq))[0] = header.StartSweepFreq;
  mxSetField(out_argv[1], 0, "StartSweepFreq", StartSweepFreq);

  EndSweepFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(EndSweepFreq)) ((short*)mxGetData(EndSweepFreq))[0] = header.EndSweepFreq;
  mxSetField(out_argv[1], 0, "EndSweepFreq", EndSweepFreq);

  SweepLength = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SweepLength)) ((short*)mxGetData(SweepLength))[0] = header.SweepLength;
  mxSetField(out_argv[1], 0, "SweepLength", SweepLength);

  SweepType = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SweepType)) ((short*)mxGetData(SweepType))[0] = header.SweepType;
  mxSetField(out_argv[1], 0, "SweepType", SweepType);

  StartSweepTaperLength = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(StartSweepTaperLength)) ((short*)mxGetData(StartSweepTaperLength))[0] = header.StartSweepTaperLength;
  mxSetField(out_argv[1], 0, "StartSweepTaperLength", StartSweepTaperLength);

  EndSweepTaperLength = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(EndSweepTaperLength)) ((short*)mxGetData(EndSweepTaperLength))[0] = header.EndSweepTaperLength;
  mxSetField(out_argv[1], 0, "EndSweepTaperLength", EndSweepTaperLength);

  TaperType = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TaperType)) ((short*)mxGetData(TaperType))[0] = header.TaperType;
  mxSetField(out_argv[1], 0, "TaperType", TaperType);

  AliasFilterFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(AliasFilterFreq)) ((short*)mxGetData(AliasFilterFreq))[0] = header.AliasFilterFreq;
  mxSetField(out_argv[1], 0, "AliasFilterFreq", AliasFilterFreq);

  AliasFilterSlope = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(AliasFilterSlope)) ((short*)mxGetData(AliasFilterSlope))[0] = header.AliasFilterSlope;
  mxSetField(out_argv[1], 0, "AliasFilterSlope", AliasFilterSlope);

  NotchFilterFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(NotchFilterFreq)) ((short*)mxGetData(NotchFilterFreq))[0] = header.NotchFilterFreq;
  mxSetField(out_argv[1], 0, "NotchFilterFreq", NotchFilterFreq);

  NotchFilterSlope = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(NotchFilterSlope)) ((short*)mxGetData(NotchFilterSlope))[0] = header.NotchFilterSlope;
  mxSetField(out_argv[1], 0, "NotchFilterSlope", NotchFilterSlope);

  LowCutFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(LowCutFreq)) ((short*)mxGetData(LowCutFreq))[0] = header.LowCutFreq;
  mxSetField(out_argv[1], 0, "LowCutFreq", LowCutFreq);

  HightCutFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(HightCutFreq)) ((short*)mxGetData(HightCutFreq))[0] = header.HightCutFreq;
  mxSetField(out_argv[1], 0, "HightCutFreq", HightCutFreq);

  LowCutSlope = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(LowCutSlope)) ((short*)mxGetData(LowCutSlope))[0] = header.LowCutSlope;
  mxSetField(out_argv[1], 0, "LowCutSlope", LowCutSlope);

  HighCutSlope = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(HighCutSlope)) ((short*)mxGetData(HighCutSlope))[0] = header.HighCutSlope;
  mxSetField(out_argv[1], 0, "HighCutSlope", HighCutSlope);

  Year = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(Year)) ((short*)mxGetData(Year))[0] = header.Year;
  mxSetField(out_argv[1], 0, "Year", Year);

  DayOfYear = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(DayOfYear)) ((short*)mxGetData(DayOfYear))[0] = header.DayOfYear;
  mxSetField(out_argv[1], 0, "DayOfYear", DayOfYear);

  Hour = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(Hour)) ((short*)mxGetData(Hour))[0] = header.Hour;
  mxSetField(out_argv[1], 0, "Hour", Hour);

  Minute = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(Minute)) ((short*)mxGetData(Minute))[0] = header.Minute;
  mxSetField(out_argv[1], 0, "Minute", Minute);

  Second = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(Second)) ((short*)mxGetData(Second))[0] = header.Second;
  mxSetField(out_argv[1], 0, "Second", Second);

  TimeCode = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TimeCode)) ((short*)mxGetData(TimeCode))[0] = header.TimeCode;
  mxSetField(out_argv[1], 0, "TimeCode", TimeCode);

  TraceWeightFactor = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TraceWeightFactor)) ((short*)mxGetData(TraceWeightFactor))[0] = header.TraceWeightFactor;
  mxSetField(out_argv[1], 0, "TraceWeightFactor", TraceWeightFactor);

  GroupRollNumber = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GroupRollNumber)) ((short*)mxGetData(GroupRollNumber))[0] = header.GroupRollNumber;
  mxSetField(out_argv[1], 0, "GroupRollNumber", GroupRollNumber);

  GroupTraceStart = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GroupTraceStart)) ((short*)mxGetData(GroupTraceStart))[0] = header.GroupTraceStart;
  mxSetField(out_argv[1], 0, "GroupTraceStart", GroupTraceStart);

  GroupTraceEnd = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GroupTraceEnd)) ((short*)mxGetData(GroupTraceEnd))[0] = header.GroupTraceEnd;
  mxSetField(out_argv[1], 0, "GroupTraceEnd", GroupTraceEnd);

  GapSize = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(GapSize)) ((short*)mxGetData(GapSize))[0] = header.GapSize;
  mxSetField(out_argv[1], 0, "GapSize", GapSize);

  Overtravel = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(Overtravel)) ((short*)mxGetData(Overtravel))[0] = header.Overtravel;
  mxSetField(out_argv[1], 0, "Overtravel", Overtravel);

  CDPEnsembleX = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(CDPEnsembleX)) ((int*)mxGetData(CDPEnsembleX))[0] = header.CDPEnsembleX;
  mxSetField(out_argv[1], 0, "CDPEnsembleX", CDPEnsembleX);

  CDPEnsembleY = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(CDPEnsembleY)) ((int*)mxGetData(CDPEnsembleY))[0] = header.CDPEnsembleY;
  mxSetField(out_argv[1], 0, "CDPEnsembleY", CDPEnsembleY);

  InLine = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(InLine)) ((int*)mxGetData(InLine))[0] = header.InLine;
  mxSetField(out_argv[1], 0, "InLine", InLine);

  CrossLine = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(CrossLine)) ((int*)mxGetData(CrossLine))[0] = header.CrossLine;
  mxSetField(out_argv[1], 0, "CrossLine", CrossLine);

  Shotpoint = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(Shotpoint)) ((int*)mxGetData(Shotpoint))[0] = header.Shotpoint;
  mxSetField(out_argv[1], 0, "Shotpoint", Shotpoint);

  ShotpointScalar = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(ShotpointScalar)) ((short*)mxGetData(ShotpointScalar))[0] = header.ShotpointScalar;
  mxSetField(out_argv[1], 0, "ShotpointScalar", ShotpointScalar);

  MeasurementUnit = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(MeasurementUnit)) ((short*)mxGetData(MeasurementUnit))[0] = header.MeasurementUnit;
  mxSetField(out_argv[1], 0, "MeasurementUnit", MeasurementUnit);

  size[0] = 1; size[1] = 6;
  TransductionConstant = mxCreateCharArray(2, size);
  if (! TransductionConstant) {
    mexPrintf("out of memory");
    return;
  }
  for (i=0; i<6; i++) 
    ((short*)mxGetPr(TransductionConstant))[i] = header.TransductionConstant[i];
  mxSetField(out_argv[1], 0, "TransductionConstant", TransductionConstant);

  TransductionUnit = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TransductionUnit)) ((short*)mxGetData(TransductionUnit))[0] = header.TransductionUnit;
  mxSetField(out_argv[1], 0, "TransductionUnit", TransductionUnit);

  DeviceID = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(DeviceID)) ((short*)mxGetData(DeviceID))[0] = header.DeviceID;
  mxSetField(out_argv[1], 0, "DeviceID", DeviceID);

  TimeScalar = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TimeScalar)) ((short*)mxGetData(TimeScalar))[0] = header.TimeScalar;
  mxSetField(out_argv[1], 0, "TimeScalar", TimeScalar);

  SourceType = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SourceType)) ((short*)mxGetData(SourceType))[0] = header.SourceType;
  mxSetField(out_argv[1], 0, "SourceType", SourceType);

  size[0] = 1; size[1] = 6;
  SourceDirection = mxCreateCharArray(2, size);
  if (! SourceDirection) {
    mexPrintf("out of memory");
    return;
  }
  for (i=0; i<6; i++) 
    ((short*)mxGetPr(SourceDirection))[i] = header.SourceDirection[i];
  mxSetField(out_argv[1], 0, "SourceDirection", SourceDirection);

  size[0] = 1; size[1] = 6;
  SourceMeasurement = mxCreateCharArray(2, size);
  if (! SourceMeasurement) {
    mexPrintf("out of memory");
    return;
  }
  for (i=0; i<6; i++) 
    ((short*)mxGetPr(SourceMeasurement))[i] = header.SourceMeasurement[i];
  mxSetField(out_argv[1], 0, "SourceMeasurement", SourceMeasurement);

  SourceMeasurementUnit = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SourceMeasurementUnit)) ((short*)mxGetData(SourceMeasurementUnit))[0] = header.SourceMeasurementUnit;
  mxSetField(out_argv[1], 0, "SourceMeasurementUnit", SourceMeasurementUnit);

  size[0] = 1; size[1] = 8;
  unassigned = mxCreateCharArray(2, size);
  if (! unassigned) {
    mexPrintf("out of memory");
    return;
  }
  for (i=0; i<8; i++) 
    ((short*)mxGetPr(unassigned))[i] = header.unassigned[i];
  mxSetField(out_argv[1], 0, "unassigned", unassigned);
}
