#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : int    = open file descriptor
//   in_argv[1] : int    = data format code, 1 (segy) or 5 (ieee big-endian)
//   in_argv[2] : struct = trace header struct
//   in_argv[3] : float * = vector of float trace data
//   out_argv[0]: int = error code, 0 on success, nonzero on error
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int fd; int format = 0; int err = 0; int len; int i;
  segy_trace_header header;
  mxArray *field;
  float *data = NULL;

  // check input arguments
  if (in_argc < 4 || ! mxIsInt32(in_argv[0]) || ! mxIsInt32(in_argv[1]) ||
      ! mxIsStruct(in_argv[2]) || mxIsDouble(in_argv[3]) 
      || mxIsEmpty(in_argv[0]) || mxIsEmpty(in_argv[1]) 
      || mxIsEmpty(in_argv[2]) || mxIsEmpty(in_argv[3])) {
    mexPrintf("segy_write_trace: input is int, int, header, data\n");
    return;
  }

  // get file descriptor
  fd = ((int*)mxGetPr(in_argv[0]))[0];

  // get data format code
  format = ((int*)mxGetPr(in_argv[1]))[0];

  // check output arguments
  if (out_argc < 1) {
    mexPrintf("segy_write_trace: must have one output argument\n");
    return;
  }

  // Copy matlab array into C data buffer
  len = mxGetNumberOfElements(in_argv[3]);
  data = (float*)malloc(sizeof(float) * len);
  for (i=0; i<len; i++) {
    data[i] = (float)(((float*)mxGetPr(in_argv[3]))[i]);
  }

  // Copy matlab struct into C struct
  if ( (field = mxGetField(in_argv[2], 0, "LineSeqNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.LineSeqNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field LineSeqNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "ReelSeqNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ReelSeqNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field ReelSeqNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "FieldRecordNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.FieldRecordNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field FieldRecordNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TraceNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TraceNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field TraceNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "EnergySourceNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.EnergySourceNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field EnergySourceNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "CDPEnsembleNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CDPEnsembleNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field CDPEnsembleNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "CDPTraceNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CDPTraceNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field CDPTraceNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TraceCode"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TraceCode = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TraceCode invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "VertSumTraces"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.VertSumTraces = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field VertSumTraces invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "HorizSumTraces"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.HorizSumTraces = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field HorizSumTraces invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "DataUse"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.DataUse = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field DataUse invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceGroupDist"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceGroupDist = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceGroupDist invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupElevation"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupElevation = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupElevation invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceElevation"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceElevation = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceElevation invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceDepth"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceDepth = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceDepth invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupDatumElevation"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupDatumElevation = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupDatumElevation invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceDatumElevation"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceDatumElevation = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceDatumElevation invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceWaterDepth"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceWaterDepth = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceWaterDepth invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupWaterDepth"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupWaterDepth = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupWaterDepth invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "ElevationScalar"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ElevationScalar = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field ElevationScalar invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "CoordinateScalar"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CoordinateScalar = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field CoordinateScalar invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceX"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceX = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceX invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceY"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceY = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceY invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupX"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupX = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupX invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupY"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupY = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupY invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "CoordinateUnit"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CoordinateUnit = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field CoordinateUnit invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "WeatheringVelocity"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.WeatheringVelocity = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field WeatheringVelocity invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SubweatheringVelocity"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SubweatheringVelocity = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SubweatheringVelocity invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceTime"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceTime = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceTime invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupTime"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupTime = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupTime invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceCorrection"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceCorrection = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceCorrection invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupCorrection"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupCorrection = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupCorrection invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TotalCorrection"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TotalCorrection = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TotalCorrection invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "LagTimeA"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.LagTimeA = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field LagTimeA invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "LagTimeB"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.LagTimeB = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field LagTimeB invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "DelayAccordingTime"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.DelayAccordingTime = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field DelayAccordingTime invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "BruteTime"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.BruteTime = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field BruteTime invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "MuteTime"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.MuteTime = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field MuteTime invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SamplesPerTrace"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SamplesPerTrace = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SamplesPerTrace invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SampleInterval"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SampleInterval = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SampleInterval invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GainType"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GainType = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GainType invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GainConstant"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GainConstant = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GainConstant invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "InitialGain"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.InitialGain = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field InitialGain invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "Correlated"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.Correlated = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field Correlated invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "StartSweepFreq"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.StartSweepFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field StartSweepFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "EndSweepFreq"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.EndSweepFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field EndSweepFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SweepLength"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SweepLength = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SweepLength invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SweepType"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SweepType = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SweepType invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "StartSweepTaperLength"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.StartSweepTaperLength = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field StartSweepTaperLength invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "EndSweepTaperLength"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.EndSweepTaperLength = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field EndSweepTaperLength invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TaperType"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TaperType = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TaperType invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "AliasFilterFreq"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.AliasFilterFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field AliasFilterFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "AliasFilterSlope"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.AliasFilterSlope = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field AliasFilterSlope invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "NotchFilterFreq"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.NotchFilterFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field NotchFilterFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "NotchFilterSlope"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.NotchFilterSlope = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field NotchFilterSlope invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "LowCutFreq"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.LowCutFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field LowCutFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "HightCutFreq"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.HightCutFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field HightCutFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "LowCutSlope"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.LowCutSlope = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field LowCutSlope invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "HighCutSlope"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.HighCutSlope = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field HighCutSlope invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "Year"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.Year = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field Year invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "DayOfYear"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.DayOfYear = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field DayOfYear invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "Hour"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.Hour = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field Hour invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "Minute"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.Minute = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field Minute invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "Second"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.Second = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field Second invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TimeCode"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TimeCode = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TimeCode invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TraceWeightFactor"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TraceWeightFactor = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TraceWeightFactor invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupRollNumber"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupRollNumber = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupRollNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupTraceStart"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupTraceStart = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupTraceStart invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GroupTraceEnd"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GroupTraceEnd = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GroupTraceEnd invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "GapSize"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.GapSize = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field GapSize invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "Overtravel"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.Overtravel = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field Overtravel invalid");
    return;
  }


  if ( (field = mxGetField(in_argv[2], 0, "CDPEnsembleX"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CDPEnsembleX = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field CDPEnsembleX invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "CDPEnsembleY"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CDPEnsembleY = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field CDPEnsembleY invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "InLine"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.InLine = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field InLine invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "CrossLine"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CrossLine = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field CrossLine invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "Shotpoint"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.Shotpoint = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field Shotpoint invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "ShotpointScalar"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ShotpointScalar = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field ShotpointScalar invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "MeasurementUnit"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.MeasurementUnit = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field MeasurementUnit invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TransductionConstant"))
       && ! mxIsEmpty(field) && mxIsChar(field)) {
    for (i=0; i<6; i++) 
      header.TransductionConstant[i] = ((short*)mxGetPr(field))[i]; 
  }
  else {
    mexPrintf("Error: field TransductionConstant invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TransductionUnit"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TransductionUnit = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TransductionUnit invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "DeviceID"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.DeviceID = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field DeviceID invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "TimeScalar"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TimeScalar = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TimeScalar invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceType"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceType = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceType invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceDirection"))
       && ! mxIsEmpty(field) && mxIsChar(field)) {
    for (i=0; i<6; i++) 
      header.SourceDirection[i] = ((short*)mxGetPr(field))[i]; 
  }
  else {
    mexPrintf("Error: field SourceDirection invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceMeasurement"))
       && ! mxIsEmpty(field) && mxIsChar(field)) {
    for (i=0; i<6; i++) 
      header.SourceMeasurement[i] = ((short*)mxGetPr(field))[i]; 
  }
  else {
    mexPrintf("Error: field SourceMeasurement invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "SourceMeasurementUnit"))
       && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SourceMeasurementUnit = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SourceMeasurementUnit invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[2], 0, "unassigned"))
       && ! mxIsEmpty(field) && mxIsChar(field)) {
    for (i=0; i<8; i++) 
      header.unassigned[i] = ((short*)mxGetPr(field))[i]; 
  }
  else {
    mexPrintf("Error: field unassigned invalid");
    return;
  }


  // call function, if error return immeadiately
  err = segy_write_trace(fd, format, &header, data, len);

  // store error return
  out_argv[0] = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (out_argv[0]) ((int*)mxGetPr(out_argv[0]))[0] = err;
  else {
    mexPrintf("segy_write_trace: out of memory\n");
    return;
  }

  if (err) mexPrintf(segy_error_message(err));
  
  free(data);
  return;
}
