#include <stdio.h>      // printf
#include <sys/types.h>  // open
#include <sys/stat.h>   // open
#include <fcntl.h>      // open
#include <unistd.h>     // close
#include <math.h>       // pow
#include <libsegy.h>

#ifndef O_BINARY
#define O_BINARY 0
#endif


// host_little_endian()
// returns 1 if (executing) host is little endian, 0 otherwise
static int host_little_endian()
{
  int a = 256;
  char *bytes = (char*)&a;
  if ((bytes[1] & 0x01) == 1) return 1;
  return 0;
}


// swap_int(i)
//   i: int * = pointer to int
// swaps byte-order of int between little/big endian
static void swap_int(int *i) {
  char *bytes = (char*)i;
  char buf = bytes[0];
  bytes[0] = bytes[3];
  bytes[3] = buf;
  buf = bytes[1];
  bytes[1] = bytes[2];
  bytes[2] = buf;
}


float segyval(float *segy)
{
  char *bytes = (char*)segy;
  
  // get sign
  int sign = (bytes[0] & 0x80) ? -1 : 1;

  // get excess-64 exponent
  char cexp[4] = { '\0', '\0', '\0', (bytes[0] & 0x7f) };
  int exp = ((int*)cexp)[0];
  if (host_little_endian()) swap_int(&exp);
  exp = (exp - 64) * 4;

  // accumulate mantissa value
  float val = 0.0;
  int byte;
  int bit;
  for (byte = 1; byte < 4; byte++) {
    for (bit = 0; bit < 8; bit++) {
      double bitexp = exp - (((byte-1)*8) + bit + 1);
      if (bytes[byte] & (0x01 << (7 - bit))) 
	val += (float)pow(2.0, bitexp);
    }
  }
  val *= sign;
  return (float)val;
}


static char hex[9] = {0};

const char * printbyte(char *byte) 
{
  char lo = (*byte) & 0x0f;
  char hi = ((*byte) >> 4) & 0x0f;
  if (lo < 10) lo += 48;
  else lo += 87;
  if (hi < 10) hi += 48;
  else hi += 87;
  hex[0] = hi;
  hex[1] = lo;
  hex[2] = 0;
  return hex;
}

// printbits(byte, buf, len)
//   byte: char   = a byte of data
//   buf : char * = string buffer
//   len : size_t = length of buf in characters
// prints string representation of byte to buf, up to len-1 characters 
void printbits(char byte, char *buf, size_t len)
{
  int val[8] = {0};
  val[0] = (byte & 0x01) ? 1 : 0;
  val[1] = (byte & 0x02) ? 1 : 0;
  val[2] = (byte & 0x04) ? 1 : 0;
  val[3] = (byte & 0x08) ? 1 : 0;
  val[4] = (byte & 0x10) ? 1 : 0;
  val[5] = (byte & 0x20) ? 1 : 0;
  val[6] = (byte & 0x40) ? 1 : 0;
  val[7] = (byte & 0x80) ? 1 : 0;
  snprintf(buf, len, "%d%d%d%d%d%d%d%d", val[7], val[6], val[5], val[4],
	   val[3], val[2], val[1], val[0]);
}


// printfloat(f, buf, len)
//   f  : float * = data value
//   buf: char *  = bit string buffer
//   len: int = length of buf
// prints bits of float in native machine order to buf string
void printfloat(float *f, char *buf, size_t len) 
{
  char *byte = (char*)f;
  char bits[33] = {0};
  printbits(byte[0], bits, 9);
  printbits(byte[1], bits+8, 9);
  printbits(byte[2], bits+16, 9);
  printbits(byte[3], bits+24, 9);

  snprintf(buf, len, "%32s", bits); 
}


// printieee(f, buf, len)
//   f  : float * = data value (little-endian)
//   buf: char *  = string buffer
//   len: int     = length of buf
// prints description of ieee bits into string buf
void printieee(float *f, char *buf, int len)
{
  char bits[33] = {0};
  char rev[33] = {0};
  char des[80] = {0};
  printfloat(f, bits, 33);
  int i, byte;
  for (byte = 3; byte >= 0; byte--) {
    for (i = 0; i < 8; i++) rev[(8*(3 - byte)) + i] = bits[(8*byte) + i];
  }

  snprintf(des, 80, "s%cs e%c%c%c%c%c%c%c%ce m%sm", rev[0], rev[1], rev[2],
	   rev[3], rev[4], rev[5], rev[6], rev[7], rev[8], &rev[9]);
  snprintf(buf, len, "%s", des);
}


// printsegy(f, buf, len)
//   f  : float * = data value (big-endian)
//   buf: char *  = string buffer
//   len: int     = length of buf
// prints description of segy bits into string buf
void printsegy(float *f, char *buf, int len)
{
  char bits[33] = {0};
  char des[80] = {0};
  printfloat(f, bits, 33);

  snprintf(des, 80, "s%cs e%c%c%c%c%c%c%ce m%sm", bits[0], bits[1], bits[2],
	   bits[3], bits[4], bits[5], bits[6], bits[7], &bits[8]);
  snprintf(buf, len, "%s", des);
}


// try_read(fd, buf, len)
//   fd : int    = file descriptor
//   buf: char * = read buffer 
//   len: int    = bytes to read
// reads len bytes of data from fd into buf, returns bytes read or -1 on fail
static int try_read(int fd, char *buf, int len)
{
  int bytes_read = 0;
  int tries = 0;
  while (bytes_read < len && tries < 10) {
    off_t prev_pos = lseek(fd, 0, SEEK_CUR);
    int err = read(fd, buf + bytes_read, len - bytes_read);
    if (err == -1) return -1;
    if (err == 0) break;
    off_t cur_pos = lseek(fd, 0, SEEK_CUR);    
    err = cur_pos - prev_pos;
    bytes_read += err;
    tries++;
  }
  return bytes_read;
}


int main(int argc, char **argv)
{
  int err = 0; 
  int i;
  int trace_num = 0;
  char buf1[4336] = {0};
  char buf2[4336] = {0};
  char *USAGE="USAGE: diffsegy segy_file1 segy_file2\n";
  if (argc < 3) {
    printf(USAGE);
    return 1;
  }

  // open files
  int fd1 = open(argv[1], O_RDONLY | O_BINARY);
  int fd2 = open(argv[2], O_RDONLY | O_BINARY);
  if (fd1 < 0 || fd2 < 0) {
    if (fd1 < 0) printf("libsegy: Error opening file %s\n", argv[1]);
    if (fd2 < 0) printf("libsegy: Error opening file %s\n", argv[2]);
    return 1;
  }

  // compare file bytes
  off_t size = lseek(fd1, 0, SEEK_END);
  lseek(fd1, 0, SEEK_SET);
  lseek(fd2, 0, SEEK_SET);

  // read file headers
  if ( (try_read(fd1, buf1, 3600) < 3600) ||
       (try_read(fd2, buf2, 3600) < 3600)) {
    printf("Error reading file header.\n");
    return 1;
  }
  
  // compare file headers
  for (i = 0; i < 3600; i++) {
    if (buf1[i] != buf2[i] && i > 3200) {
      printf("File headers not equal: byte %d ", i);
      printf("file1(%s) ", printbyte(&buf1[i]));
      printf("file2(%s)\n", printbyte(&buf2[i]));
    }
  }

  // read traces
  for (i = 0; i < size/4336; i++) {
    if ( (err = try_read(fd1, buf1, 4336)) < 4336) {
      if (err == 0) {
	printf("end of file1 read.\n");
	return 0;
      }
      else {
	printf("Read error.\n");
	return 1;
      }
    }
    if ( (err = try_read(fd2, buf2, 4336)) < 4336) {
      if (err == 0) {
	printf("end of file2 read.\n");
	return 0;
      }
      else {
	printf("Read error.\n");
	return 1;
      }
    }
    trace_num++;

    // compare trace headers
    int j;
    for (j = 0; j < 240; j++) {
      if (buf1[j] != buf2[j]) {
	printf("Trace %d headers not equal: byte %d ", trace_num, j);
	printf("file1(%s) ", printbyte(&buf1[j]));
	printf("file2(%s)\n", printbyte(&buf2[j]));
      }
    }

    // compare trace data
    int datum = 1;
    for (j = 240; j < 4336; j+=4) {
      int printed = 0;
      int k;
      for (k = j; k < j+4 && ! printed; k++) {
	if (buf1[k] != buf2[k]) {
	  float ieee;
	  char str[80] = {0};

	  printf("Trace %d datum %d not equal:\n", trace_num, datum);
	  printsegy((float*)&buf1[j], str, 80);
	  printf("segy1: %s (%+1.8e)\n", str, segyval((float*)&buf1[j]));
	  convert2ieee(1, (char*)&buf1[j], 4, &ieee, 1);
	  printieee(&ieee, str, 80);
	  printf("ieee1: %s (%+1.8e)\n", str, ieee);

	  printsegy((float*)&buf2[j], str, 80);
	  printf("segy2: %s (%+1.8e)\n", str, segyval((float*)&buf2[j]));
	  convert2ieee(1, (char*)&buf2[j], 4, &ieee, 1);
	  printieee(&ieee, str, 80);
	  printf("ieee2: %s (%+1.8e)\n\n", str, ieee); 
	  printed = 1;
	}
      }
      datum++;
    }
  }

  return 0;
}
